
/* Chrysalide - Outil d'analyse de fichiers binaires
 * link.c - édition des liens après la phase de désassemblage
 *
 * Copyright (C) 2014 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "link.h"


#include <assert.h>


#include "target.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : instr   = instruction ARMv7 à traiter.                       *
*                proc    = représentation de l'architecture utilisée.         *
*                context = contexte associé à la phase de désassemblage.      *
*                format  = acès aux données du binaire d'origine.             *
*                                                                             *
*  Description : Etablit un lien de saut selon une instruction donnée.        *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void handle_jump_as_link(GArchInstruction *instr, GArchProcessor *proc, GProcContext *context, GExeFormat *format)
{
    GArchOperand *op;                       /* Opérande numérique en place */
    virt_t virt;                            /* Adresse virtuelle           */
    vmpa2t addr;                            /* Adresse de destination      */
    GArchInstruction *target;               /* Ligne visée par la référence*/

    assert(g_arch_instruction_count_operands(instr) > 0);

    op = g_arch_instruction_get_operand(instr, 0);

    if (!G_IS_IMM_OPERAND(op)) return;

    if (g_imm_operand_to_virt_t(G_IMM_OPERAND(op), &virt))
    {
        init_vmpa(&addr, VMPA_NO_PHYSICAL, virt);

        target = g_arch_processor_find_instr_by_address(proc, &addr);

        if (target != NULL)
        {
            g_arch_instruction_link_with(instr, target, ILT_JUMP);
            g_object_unref(G_OBJECT(target));
        }

    }

}


/******************************************************************************
*                                                                             *
*  Paramètres  : instr   = instruction ARMv7 à traiter.                       *
*                proc    = représentation de l'architecture utilisée.         *
*                context = contexte associé à la phase de désassemblage.      *
*                format  = acès aux données du binaire d'origine.             *
*                index   = indice de l'opérande à traiter dans l'instruction. *
*                                                                             *
*  Description : Etablit un lien d'appel selon une instruction donnée.        *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void handle_branch_as_link(GArchInstruction *instr, GArchProcessor *proc, GProcContext *context, GExeFormat *format, size_t index)
{
    GArchOperand *op;                       /* Opérande numérique en place */
    virt_t virt;                            /* Adresse virtuelle           */
    vmpa2t addr;                            /* Adresse de destination      */
    instr_iter_t *iter;                     /* Parcours d'instructions     */
    GArchInstruction *target;               /* Ligne visée par la référence*/

    assert(g_arch_instruction_count_operands(instr) > index);

    op = g_arch_instruction_get_operand(instr, index);

    virt = VMPA_NO_VIRTUAL;

    if (G_IS_TARGET_OPERAND(op))
        virt = g_target_operand_get_addr(G_TARGET_OPERAND(op));

    else if (G_IS_IMM_OPERAND(op))
    {
        if (!g_imm_operand_to_virt_t(G_IMM_OPERAND(op), &virt))
            virt = VMPA_NO_VIRTUAL;
    }

    if (virt != VMPA_NO_VIRTUAL)
    {
        init_vmpa(&addr, VMPA_NO_PHYSICAL, virt);

        iter = g_arch_processor_get_iter_from_address(proc, &addr);

        if (iter != NULL)
        {
            target = get_instruction_iterator_current(iter);

            if (target != NULL)
            {
                g_arch_instruction_link_with(instr, target, ILT_JUMP_IF_TRUE);

                g_object_unref(G_OBJECT(target));

            }

            target = get_instruction_iterator_next(iter);

            if (target != NULL)
            {
                g_arch_instruction_link_with(instr, target, ILT_JUMP_IF_FALSE);

                g_object_unref(G_OBJECT(target));

            }

            delete_instruction_iterator(iter);

        }

    }

}


/******************************************************************************
*                                                                             *
*  Paramètres  : instr   = instruction ARMv7 à traiter.                       *
*                proc    = représentation de l'architecture utilisée.         *
*                context = contexte associé à la phase de désassemblage.      *
*                format  = acès aux données du binaire d'origine.             *
*                                                                             *
*  Description : Etablit un lien d'appel selon une instruction donnée.        *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void handle_call_as_link(GArchInstruction *instr, GArchProcessor *proc, GProcContext *context, GExeFormat *format)
{
    GArchOperand *op;                       /* Opérande numérique en place */
    virt_t virt;                            /* Adresse virtuelle           */
    vmpa2t addr;                            /* Adresse de destination      */
    GArchInstruction *target;               /* Ligne visée par la référence*/

    assert(g_arch_instruction_count_operands(instr) > 0);

    op = g_arch_instruction_get_operand(instr, 0);

    if (!G_IS_IMM_OPERAND(op)) return;

    if (g_imm_operand_to_virt_t(G_IMM_OPERAND(op), &virt))
    {
        init_vmpa(&addr, VMPA_NO_PHYSICAL, virt);

        target = g_arch_processor_find_instr_by_address(proc, &addr);

        if (target != NULL)
        {
            g_arch_instruction_link_with(instr, target, ILT_CALL);
            g_object_unref(G_OBJECT(target));
        }

    }

}
